package net.mummila.crosswordgame;
import java.util.Random;
import java.util.Vector;

import org.eclipse.swt.widgets.Button;



// all the letters (the letter "pieces") available in a game.
public class CGAlphabet {
	
	// the signal for "no more letters available".
	// must be a single character (but still a String, not a Character)
	// which is not used as a letter in the game.
	// CGLetterSet needs to know about this.
	final public static String NOLETTER = "?";

	// the letters are stored in a Vector of Strings.
	final private Vector<String> alphabet = new Vector<String>();
	
	
	
	// the constructor sets the parent game.
	final private CrosswordGame game;
	public CGAlphabet(final CrosswordGame g) {
		game = g;
		initialize();
	}
	
	// at initialization, the alphabet are loaded from a file, if available.
	// otherwise a default set of letters is used.
	// the letters are scrambled so they can be written in order in the file, if desired.
	public void initialize() {
		set(game.settings.getAlphabet());
		scramble();
	}
	
	// return the alphabet as a String.
	// mainly for use in communication with the opponent.
	public String toString() {
		String s = "";
		for (int i=0; i<alphabet.size(); i++) {
			s = s + alphabet.elementAt(i);
		}
		return s;
	}
	
	// set the alphabet according to a given String.
	// note that the String can be empty (""), though not null.
	// returns FALSE if the String is null.
	public boolean set(final String s) {

		// test for given String's validity.
		if (s == null)				return false;
		
		// clear away the old alphabet, if any.
		alphabet.removeAllElements();

		// load in the given letters.
		for (int i=0; i<s.length(); i++) {
			alphabet.addElement(s.substring(i, i+1));
		}
		
		return true;
	}
	
	// scrambling the alphabet.
	private void scramble() {
		
		// the random number generator.
		Random r = new Random();

		// scrambling is done by
		// going through each letter in turn,
		// switching it with a randomly selected partner.
		for (int i=0; i<alphabet.size(); i++) {
			int random = r.nextInt(alphabet.size());
			String s = alphabet.elementAt(random);
			String t = alphabet.elementAt(i);
			alphabet.set(i, s);
			alphabet.set(random, t);
		}
	}
	
	// picking up a letter.
	// returns NOLETTER to signal failure when no more letters are available.
	public String pick() {
		
		if (alphabet.size() > 0) {
			String pick = alphabet.elementAt(0); 
			alphabet.remove(0);
			return pick;
		}
		
		else	return NOLETTER;
 	}
	
	// push all the player's letters back into the alphabet.
	public void pushBackAll() {
		
		for (int i=0; i<game.playerSlots.size(); i++) {
			alphabet.add(game.playerSlots.elementAt(i).getText());
			game.playerSlots.elementAt(i).clear();
		}
		
		// the Alphabet are (re)scrambled not to make the
		// pushed-back letters appear in the same order
		// at the end of a game.
		scramble();
	}

	// push a given letter back into the alphabet.
	public void pushBack(Button letter) {

		// a sanity check.
		if (letter == null)	return;
		
		alphabet.add(0, letter.getText());
		letter.setText("");
	}
	
	// push contents of a given letter slot back into the alphabet.
	public void pushBack(CGLetterSlot s) {

		// a sanity check.
		if (s == null)	return;
		
		alphabet.add(0, s.getLetter().toString());
		s.setLetter(new CGLetter());
	}
	
	// a wrapper for the size of the alphabet.
	public int size() {
		return alphabet.size();
	}
}
