package net.mummila.crosswordgame;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;



public class CGLetterSlot extends Composite implements MouseListener, SelectionListener {

	final public Button button;
	
	private CGLetter letter;
	
	final private CrosswordGame game;
	public CGLetterSlot(final Composite parent, final CrosswordGame g) {
		super(parent, SWT.BORDER);
		game = g;
		
		GridLayout gl = new GridLayout(1, true);
		gl.marginHeight = 0;
		gl.marginWidth = 0;
		GridData gd = new GridData();
		gd.widthHint = 23;
		gd.heightHint = 23;
		this.setLayout(gl);
		this.setLayoutData(gd);

		button = new Button(this, SWT.PUSH);
		GridData gdbutton = new GridData();
		gdbutton.widthHint = 23;
		gdbutton.heightHint = 23;
		button.setLayoutData(gdbutton);
		button.setVisible(false);
		setLetter(new CGLetter());

		this.addMouseListener(this);
		button.addSelectionListener(this);
	}
	public void clear() {	initialize();	}
	public void initialize() {
		setLetter(new CGLetter());
	}
	public void disable() {
		button.setEnabled(false);
		this.setEnabled(false);
	}
	public void enable() {
		button.setEnabled(true);
		this.setEnabled(true);
	}
	public void setLetter(CGLetter l) {
		letter = l;
		button.setImage(letter.getLetterImage());
		if (letter.isEmpty())	{
			button.setVisible(false);
		}
		else {
			button.setVisible(true);
			if (this.getEnabled())	button.setEnabled(true);
			else					button.setEnabled(false);
		}
	}
	public CGLetter getLetter() {
		return letter;
	}
	public boolean isEmpty() {
		return (letter.isEmpty());
	}
	
	// deposit a letter into an empty slot.
	public void mouseDoubleClick(MouseEvent e) {	}
	public void mouseDown(MouseEvent e) {	}
	public void mouseUp(MouseEvent e) {
		if (!game.queue.isEmpty())	send(this);
		if (this instanceof CGCell)	((CGCell) this).highlight();
		setLetter(game.queue.get());
	}
	
	// letter pickup / swap with queue.
	public void widgetSelected(SelectionEvent e) {	widgetDefaultSelected(e);	}
	public void widgetDefaultSelected(SelectionEvent e) {
		if (!game.queue.isEmpty())	send(this);
		game.queue.swap(this);
	}
	
	public String getText() {
		return letter.toString();
	}
	
	private void send(CGLetterSlot s) {
		if ( (game.connection.connected) && game.isMyTurn ) {
			String moveType = CGProtocol.MOVE_TYPES_O2O;
			if (game.queue.isCell() && s instanceof CGCell)	moveType = CGProtocol.MOVE_TYPES_C2C;
			else if (game.queue.isCell())					moveType = CGProtocol.MOVE_TYPES_C2O;
			else if (s instanceof CGCell)				moveType = CGProtocol.MOVE_TYPES_O2C;
			final CGMove sendMove = new CGMove(game, game.queue.slot, s, moveType);
			if (game.connection.sendMove(sendMove))	game.setStatus("");
			else									game.disconnected();
		}
		else										game.setStatus("");
	}
}
