package net.mummila.crosswordgame;

import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;



//the menus of the main window. 
public class CGMenu {

	// the menus and their items.
	final private static String MENU_GAME								= "&Peli";
	final private static String MENU_GAME_ITEM_DICTIONARY				=	"&Sanasto";
	final private static String MENU_GAME_ITEM_UNDO						=	"&Kumoa";
	final private static String MENU_GAME_ITEM_RESET					=	"N&ollaa";
	final private static String MENU_GAME_ITEM_QUIT						=	"&Lopeta";
	final private static String MENU_GAME_ITEM_EXIT						=	"S&ammuta";
	final private static String MENU_SETTINGS							= "As&etukset";
	final private static String MENU_SETTINGS_MENU_PLAYER				=	"Pelaa&ja";
	final private static String MENU_SETTINGS_MENU_PLAYER_ITEM_PLAYER1	=		"Jan&i";
	final private static String MENU_SETTINGS_MENU_PLAYER_ITEM_PLAYER2	=		"Jann&e";
	final private static String MENU_SETTINGS_ITEM_ADDRESS				=	"&Vastustajan IP-osoite";
	
	// the menu bar itself. the disabler/enabler need to see it.
	private Menu menuBar;

	// the Game > Exit confirmation dialog's contents.
	// which of the two messages is displayed is determined by
	// whether a connection is up when the confirmation is represented.
	final private static String EXIT_CONFIRMATION_TITLE	 = "Varmistus: pelin sammuttaminen";
	final private static String EXIT_CONFIRMATION_MESSAGE_CONNECTED =
		"Pelin sammuttaminen merkitsee sit, ett\n" +
		"yhteys vastapelaajaan katkaistaan ja\n" +
		"tiedot pelilaudalla olevista kirjaimista\n" +
		"menetetn.\n" +
		"\n" +
		"Oletko varma ett haluat sammuttaa pelin?";
	final private static String EXIT_CONFIRMATION_MESSAGE =
		"Pelin sammuttaminen merkitsee sit, ett\n" +
		"tiedot pelilaudalla olevista kirjaimista\n" +
		"menetetn.\n" +
		"\n" +
		"Oletko varma ett haluat sammuttaa pelin?";

	// the status message after having quit a game.
	// the _SAVED version is for announcing that the final score was saved,
	// the _FAILED one for when the saving failed.
	final private static String QUIT_STATUS			= "Peli pttyi.";
	final private static String QUIT_STATUS_SAVED	= "Pisteet tallennettiin. Peli pttyi.";
	final private static String QUIT_STATUS_SAVE_FAILED	= "Pisteiden tallentaminen eponnistui.";
	
	// these need to be seen by their disabler/enabler.
	final private MenuItem undo;
	final private MenuItem quit;
	final private MenuItem dictionary;
	
	// the player selector items need to be seen by their listener.
    final private MenuItem player1;
    final private MenuItem player2;

	// the constructor parameters are the main window and the parent game.
	final private CrosswordGame game;
	final private Shell shell;
	public CGMenu(final Shell s, final CrosswordGame g) {
		shell = s;
		game = g;
		
		menuBar = new Menu(shell, SWT.BAR);

		// Game
		// - Dictionary
		// - Undo
		// - Reset
	    // `- Exit
		MenuItem gameHeader = new MenuItem(menuBar, SWT.CASCADE);
		gameHeader.setText(MENU_GAME);
		Menu gameMenu = new Menu(shell, SWT.DROP_DOWN);
		gameHeader.setMenu(gameMenu);
		
	    dictionary = new MenuItem(gameMenu, SWT.PUSH);
	    dictionary.setText(MENU_GAME_ITEM_DICTIONARY);
	    dictionary.addSelectionListener(new dictionaryListener());
	    
	    undo = new MenuItem(gameMenu, SWT.PUSH);
	    undo.setText(MENU_GAME_ITEM_UNDO);
	    undo.addSelectionListener(new undoListener());
	    
	    new MenuItem(gameMenu, SWT.SEPARATOR);

	    final MenuItem reset = new MenuItem(gameMenu, SWT.PUSH);
	    reset.setText(MENU_GAME_ITEM_RESET);
	    reset.addSelectionListener(new resetListener());
	    
	    quit = new MenuItem(gameMenu, SWT.PUSH);
	    quit.setText(MENU_GAME_ITEM_QUIT);
	    quit.addSelectionListener(new quitListener());
	    
	    final MenuItem exit = new MenuItem(gameMenu, SWT.PUSH);
	    exit.setText(MENU_GAME_ITEM_EXIT);
	    exit.addSelectionListener(new exitListener());
	    
	    // Settings
		// `- Player
	    //    `- Player 1
	    //    `- Player 2
	    // `- Opponent's address

		MenuItem settingsHeader = new MenuItem(menuBar, SWT.CASCADE);
		settingsHeader.setText(MENU_SETTINGS);
		Menu settingsMenu = new Menu(shell, SWT.DROP_DOWN);
		settingsHeader.setMenu(settingsMenu);

		final MenuItem player = new MenuItem(settingsMenu, SWT.CASCADE);
	    player.setText(MENU_SETTINGS_MENU_PLAYER);
	    final Menu playerMenu = new Menu(shell, SWT.DROP_DOWN);
	    player.setMenu(playerMenu);
	    
	    player1 = new MenuItem(playerMenu, SWT.RADIO);
	    player1.setText(MENU_SETTINGS_MENU_PLAYER_ITEM_PLAYER1);
	    player1.addSelectionListener(new playerListener());
	    player2 = new MenuItem(playerMenu, SWT.RADIO);
	    player2.setText(MENU_SETTINGS_MENU_PLAYER_ITEM_PLAYER2);
	    player2.addSelectionListener(new playerListener());
	    if (game.settings.getPlayer() == CGSettings.PLAYER2)	player2.setSelection(true);
	    else													player1.setSelection(true);

	    final MenuItem address = new MenuItem(settingsMenu, SWT.PUSH);
	    address.setText(MENU_SETTINGS_ITEM_ADDRESS);
	    address.addSelectionListener(new addressListener());

		shell.setMenuBar(menuBar);
	}

	// disable/enable the menu bar.
	public void disable() {
		//menuBar.setEnabled(false);
		for (int i=0; i<menuBar.getItemCount(); i++) {
			MenuItem item = menuBar.getItem(i);
			if (item.getMenu() != null) {
				for (int j=0; j<item.getMenu().getItemCount(); j++) {
					MenuItem jtem = item.getMenu().getItem(j);
					if ( jtem != dictionary) jtem.setEnabled(false);
				}
			}
		}
	}
	public void enable() {
		//menuBar.setEnabled(true);
		for (int i=0; i<menuBar.getItemCount(); i++) {
			MenuItem item = menuBar.getItem(i);
			if (item.getMenu() != null) {
				for (int j=0; j<item.getMenu().getItemCount(); j++) {
					item.getMenu().getItem(j).setEnabled(true);
				}
			}
		}
	}
	
	// disable/enable the 'Undo' item.
	public void disableUndo() {
		undo.setEnabled(false);
	}
	public void enableUndo() {
		undo.setEnabled(true);
	}

	// disable/enable the 'Quit' item.
	public void disableQuit() {
		quit.setEnabled(false);
	}
	public void enableQuit() {
		quit.setEnabled(true);
	}
	
	
	// the undo item's functionality.
	private class undoListener implements SelectionListener {
		public void widgetSelected(SelectionEvent event) {
			game.crossword.cancelWords(true);
			game.opponentSlots.undo();
			game.crossword.undoOpponent();
    		game.scoreTable.setAlphabetCounter();
    		
    		int minusPts = game.scoreTable.undoPts;
    		game.scoreTable.increaseOpponentScore(-minusPts);
    		
			// if connected to opponent, send the move.
			if (game.connection.connected) {
				CGMove move = new CGMove(game, CGProtocol.MOVE_TYPES_UNDOFIX, minusPts);
				game.connection.sendMove(move);
				game.connection.waitForTurn();
			}
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
	
	// the dictionary item's functionality: open up the dictionary dialog.
	private class dictionaryListener implements SelectionListener {
		public void widgetSelected(SelectionEvent event) {
			game.dictionary.open(game.crossword.getInsertedWord());
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
	
	// the reset item's functionality: shut down any connections and initialize a new game.
	private class resetListener implements SelectionListener {
		public void widgetSelected(SelectionEvent event) {
			if (game.confirmed())	game.initialize();
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}

	// the quit item's functionality: shut down connection, count the final score,
	// suggest saving the score.
	// CGMove needs to see this.
	public void gameOver() {	gameOver(false);	}
	public void gameOver(final boolean force) {
		int pInit = game.scoreTable.getPlayerFinalScore();
		int oInit = game.scoreTable.getOpponentFinalScore();
		QuitDialog qd = new QuitDialog(force, pInit, oInit, shell);
		QuitDialogResult result = qd.open();
		if (force) {
			game.hasEndedGracefully = true;
			if (result.save) {
				if (game.stats.saveStats(result.score.playerScore, result.score.opponentScore))
					game.disconnected(QUIT_STATUS_SAVED);
				else
					game.disconnected(QUIT_STATUS_SAVE_FAILED);
			}
			else	game.disconnected(QUIT_STATUS);
		}
		else {
			if (result.cancel)	return;
			game.connection.sendMove(new CGMove(game, CGProtocol.MOVE_TYPES_GAME_OVER, 0));
			game.hasEndedGracefully = true;
			if (result.save) {
				if (game.stats.saveStats(result.score.playerScore, result.score.opponentScore))
					game.disconnected(QUIT_STATUS_SAVED);
				else
					game.disconnected(QUIT_STATUS_SAVE_FAILED);
			}
			else				game.disconnected(QUIT_STATUS);
		}
	}
	private class quitListener implements SelectionListener {
		public void widgetSelected(SelectionEvent event) {
			gameOver();
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
	
	// the exit item's functionality: shut down the main window when selected.
	// exit() needs to be seen by CrosswordGame.
	public void exit() {
		try {
	    	if (game.noMovesMade() && !game.connection.connected) {
	    		game.exit(); 
	    	}
			else if (
				game.confirmed(
						EXIT_CONFIRMATION_TITLE,
						EXIT_CONFIRMATION_MESSAGE,
						EXIT_CONFIRMATION_MESSAGE_CONNECTED
				)
			) game.exit();
		}
		
		catch (SWTException ex) {
			;
		}
	}
	private class exitListener implements SelectionListener {
	    public void widgetSelected(SelectionEvent event) {
	    	exit();
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
	
	// the player selector functionality.
	private class playerListener implements SelectionListener {
	    public void widgetSelected(SelectionEvent event) {
	    	if ((MenuItem) event.widget == player1) {
	    		game.settings.setPlayer(CGSettings.PLAYER1);
	    	}
	    	else {
	    		game.settings.setPlayer(CGSettings.PLAYER2);
	    	}
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}

	// the opponent IP address functionality: open the address dialog.
	private class addressListener implements SelectionListener {
	    public void widgetSelected(SelectionEvent event) {
	    	CGAddressDialog dialog = new CGAddressDialog(shell, game);
	    	dialog.open();
	    }
		public void widgetDefaultSelected(SelectionEvent event) {
			widgetSelected(event);
		}
	}
	
	

	private class QuitDialogResult {
		final private FinalScore score;
		final public boolean cancel;
		final public boolean save;
		private QuitDialogResult(final int p, final int o, final boolean c, final boolean s) {
			score = new FinalScore(p, o);
			cancel = c;
			save = s;
		}
		private QuitDialogResult() {
			score = new FinalScore(0, 0);
			cancel = true;
			save = false;
		}
		private class FinalScore {
			final private int playerScore;
			final private int opponentScore;
			private FinalScore(final int p, final int o) {
				playerScore = p;
				opponentScore = o;
			}
		}
	}
	private class QuitDialog extends Dialog {

		// the Game > Quit confirmation dialog's contents.
		final private static String QUIT_TITLE	 = "Varmistus: pelin lopettaminen";
		final private static String QUIT_MESSAGE =
			"Pelin lopettaminen merkitsee sit, ett yhteys vastapelaajaan katkaistaan. " +
			"Voit lisksi halutessasi tallentaa lopullisen pistetilanteen " +
			"tilastointia varten.\n" +
			"\n" +
			"Pelin lopettaminen ei merkitse samaa kuin ohjelman sammuttaminen. " +
			"Mikli haluat sammuttaa ohjelman, voit tehd sen valitsemalla Peli-valikosta " +
			"kohdan 'Sammuta'. Huomaa, ett tllin pistetilannetta ei tallenneta.";
		final private static String QUIT_MESSAGE_FORCE =
			"Vastapelaaja on lopettanut pelin. " +
			"Voit halutessasi tallentaa lopullisen pistetilanteen tilastointia varten.";
		
		// dialog width.
		final private static int WIDTH	= 280;
		
		// the score input has four labels:
		// FINAL_PLAYER_SCORE	<>
		// FINAL_OPPONENT_SCORE	<>
		//   [x] SAVE_SCORE_YES
		//	 [ ] SAVE_SCORE_NO
		//
		// for example:
		// Your final score:		<>
		// Opponent's final score: <>
		//		[x] Save the score
		//		[ ] Don't save
		final private static String FINAL_SCORE_PLAYER	= "Lopulliset pisteesi:";
		final private static String FINAL_SCORE_OPPONENT= "Vastustajan pisteet:";
		final private static String SAVE_SCORE_YES		= "Pisteet tallennetaan.";
		final private static String SAVE_SCORE_NO		= "Pisteit ei tallenneta.";
		
		// reasonable minimum and maximum values for the score Spinners.
		final private static int	SCORE_MINIMUM 	= 0;
		final private static int	SCORE_MAXIMUM 	= CGScoreTable.MAXIMUM;

		final private static String BUTTONS_PROMPT =
			"Alla olevilla napeilla voit vahvistaa tai peruuttaa pelin lopettamisen " +
			"sek yll tekemsi valinnat.\n" +
			"\n" +
			"Haluatko vahvistaa yll tekemsi valinnat ja lopettaa pelin?";
		final private static String BUTTONS_PROMPT_FORCE =
			"Haluatko tallentaa nm pisteet tilastointia varten?";
		
		// the choice Button labels.
		final private static String BUTTON_LABEL_SUBMIT	= "Kyll";
		final private static String BUTTON_LABEL_CANCEL	= "Ei";

		// the results to be returned.
		private QuitDialogResult result;
		
		// the dialog window.
		private Shell shell;

		// initial score.
		final int playerInit;
		final int opponentInit;
		
		// whether the opponent already quit.
		final boolean force;
		
		private SaveSelector saveSelector;
		
		// the constructor.
		private QuitDialog(final boolean f, final int pInit, final int oInit, final Shell parent) {
			super (parent, SWT.NONE);
			force = f;
			playerInit = pInit;
			opponentInit = oInit;
		}

		

		// open the dialog.
		// on 'Submit' returns the score selected by the user.
		// on 'Cancel' returns a non-valid score. 
		public QuitDialogResult open() {

			Shell parent = getParent();
			shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
			shell.setText(getText());
			GridLayout gl = new GridLayout(1, true);
			gl.marginHeight = 20;
			gl.marginWidth = 20;
			shell.setLayout(gl);
			shell.setText(QUIT_TITLE);
			
			// set the prompt above the score Spinners.
			// wrapped in a Composite since everything else is too, so that margins are equal.
			class MainPrompt extends Composite {
				public MainPrompt() {
					super(shell, SWT.NONE);
					GridLayout gl = new GridLayout(1, false);
					this.setLayout(gl);
					final Label prompt = new Label(this, SWT.WRAP);
					if (force)	prompt.setText(QUIT_MESSAGE_FORCE);
					else		prompt.setText(QUIT_MESSAGE);
					GridData gdPrompt = new GridData();
					gdPrompt.widthHint = WIDTH;
					prompt.setLayoutData(gdPrompt);
				}
			}
			new MainPrompt();
			
			Label separator1 = new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);
			GridData fill = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
			separator1.setLayoutData(fill);
			
			// score input for modifying the final score.
			final ScoreInput playerScore = new ScoreInput(playerInit, FINAL_SCORE_PLAYER);
			final ScoreInput opponentScore = new ScoreInput(opponentInit, FINAL_SCORE_OPPONENT);
			GridData gdscore = new GridData();
			gdscore.horizontalAlignment = SWT.CENTER;
			playerScore.setLayoutData(gdscore);
			opponentScore.setLayoutData(gdscore);
			
			if (!force) {
				saveSelector = new SaveSelector();
				GridData gdss = new GridData();
				gdss.horizontalAlignment = SWT.CENTER;
				saveSelector.setLayoutData(gdss);
			}
			
			Label separator2 = new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);
			separator2.setLayoutData(fill);
			
			// set the prompt above the choice Buttons.
			// wrapped in a Composite since everything else is too, so that margins are equal.
			class ButtonPrompt extends Composite {
				public ButtonPrompt() {
					super(shell, SWT.NONE);
					GridLayout gl = new GridLayout(1, false);
					this.setLayout(gl);
					final Label prompt = new Label(this, SWT.WRAP);
					if (force)	prompt.setText(BUTTONS_PROMPT_FORCE);
					else		prompt.setText(BUTTONS_PROMPT);
					GridData gdprompt = new GridData();
					gdprompt.widthHint = WIDTH;
					prompt.setLayoutData(gdprompt);
				}
			}
			new ButtonPrompt();
			
			final Buttons buttons = new Buttons(playerScore, opponentScore, shell);
			GridData center = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
			buttons.setLayoutData(center);

			// set up an exit listener to make exit act like 'Cancel'.
			// http://www.java2s.com/Code/Java/SWT-JFace-Eclipse/Preventashellfromclosingprompttheuser.htm
		    shell.addListener(SWT.Close, new Listener() {
				public void handleEvent(Event event) {
					result = new QuitDialogResult();
				}
		    });

		    shell.pack();
			shell.open();

			Display display = parent.getDisplay();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) display.sleep();
			}
			
			return result;		
		}
		
		
		
		// score input for modifying the number of points to be awarded for the move.
		private class ScoreInput extends Composite {
			final private Spinner score;
			public ScoreInput(final int pts, final String p) {
				super(shell, SWT.NONE);
				GridLayout gl = new GridLayout(2, true);
				this.setLayout(gl);
				
				// set the prompt above the score Spinner.
				final Label prompt = new Label(this, SWT.WRAP);
				prompt.setText(p);
				GridData gdPrompt = new GridData();
				gdPrompt.horizontalAlignment = SWT.END;
				prompt.setLayoutData(gdPrompt);
			
				// the score Spinner.
				// http://www.java2s.com/Code/Java/SWT-JFace-Eclipse/NumberInputDialog.htm
			    score = new Spinner(this, SWT.BORDER);
			    score.setMinimum(SCORE_MINIMUM);
			    score.setMaximum(SCORE_MAXIMUM);
			    score.setSelection(pts);
			    score.setIncrement(1);
			    score.setPageIncrement(10);
				GridData gd = new GridData(GridData.FILL_HORIZONTAL);
				gd.horizontalAlignment = SWT.CENTER;
				score.setLayoutData(gd);
			}
			public int getScore() {
				return score.getSelection();
			}
		}
		
		// a couple of radio buttons for selecting whether to
		// save the word to the dictionary or not.
		private class SaveSelector extends Composite {
			final private Button yesButton;
			final private Button noButton;
			private SaveSelector() {
				super(shell, SWT.NONE);
				GridLayout glSelector = new GridLayout(1, false);
				this.setLayout(glSelector);
				yesButton = new Button(this, SWT.RADIO);
				yesButton.setText(SAVE_SCORE_YES);
				noButton = new Button(this, SWT.RADIO);
				noButton.setText(SAVE_SCORE_NO);
				yesButton.setSelection(true);
			}
			private boolean saveSelected() {
				return yesButton.getSelection();
			}
		}

		// the final choice buttons: 'Submit', 'Cancel'.
		private class Buttons extends Composite {
			private Buttons(final ScoreInput playerScore, final ScoreInput opponentScore, final Shell shell) {
				super(shell, SWT.NONE);
				GridLayout gl = new GridLayout(2, true);
				this.setLayout(gl);
				
				GridData gdFill = new GridData(GridData.FILL_BOTH);
				
			    final Button submit = new Button(this, SWT.PUSH);
				submit.setText(BUTTON_LABEL_SUBMIT);
				submit.setLayoutData(gdFill);
				
				gdFill = new GridData(GridData.FILL_BOTH);
				
				final Button cancel = new Button(this, SWT.PUSH);
				cancel.setText(BUTTON_LABEL_CANCEL);
				cancel.setLayoutData(gdFill);

				// react to buttons being clicked.
				submit.addListener(SWT.Selection, new Listener() {
					public void handleEvent(Event event) {
						if (force)
							result = new QuitDialogResult(playerScore.getScore(), opponentScore.getScore(), false, true);
						else
							result = new QuitDialogResult(playerScore.getScore(), opponentScore.getScore(), false, saveSelector.saveSelected());
						
						shell.dispose();
					}
				});
				cancel.addListener(SWT.Selection, new Listener() {
					public void handleEvent(Event event) {
						result = new QuitDialogResult();
						shell.dispose();
					}
				});
			}
		}
	}
}
