package net.mummila.crosswordgame;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.SocketTimeoutException;



// the server end of a connection.
public class CGServer extends CGClient {
	
	// server and client sockets.
	private ServerSocket serverSocket = null;
    private Socket clientSocket = null;

    
    
	// the constructor sets the parent game.
    // the other parameter is the opponent's IP address.
	public CGServer(final CrosswordGame game, final String ip) {
        super(game, ip);
	}

	// overrides CGClient's connect().
	public boolean connect(boolean continueGame) {
		
		// establish the ServerSocket.
		try {
	        serverSocket = new ServerSocket(PORT);
		}
		catch (IOException ex) {
			return false;
		}
        // the ServerSocket is now established.

        // set timeout for waiting client connections,
        // in cycles of 1000 ms (1 s).
    	try {
    		serverSocket.setSoTimeout(WAIT_MILLISECONDS);
        }
        
        catch (SocketException ex) {
			return false;
        }
        // the timeout for waiting client connections is now set.
        
        // wait for client connections.
		// die gracefully if there aren't any incoming connections.
    	try {
        	clientSocket = serverSocket.accept();
        }
        catch (SocketTimeoutException ex) {
			return false;
        }
		catch (IOException ex) {
        	return false;
		}
		// the client connection is now established.
		
    	// establish input and output streams.
        try {
	        out = new PrintWriter(clientSocket.getOutputStream(), true);
	        in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
        }
		catch (IOException ex) {
        	return false;
		}
        // the input and output streams are now established.
		
        // send game initialization to the client.
		return true;
	}
	
    // the method for shutting down the server connection.
	// CGConnection needs to see this.
	// overrides initialize() in CGClient.
	public void initialize() {
		try {
			if (out != null) {
				out.close();
			}
			if (in != null) {
				in.close();
			}
			if (clientSocket != null) {
				clientSocket.close();
			}
			if (serverSocket != null) {
				serverSocket.close();
			}
		}

		catch (IOException e) {
        }
		
		connected = false;
	}
}
