package net.mummila.crosswordgame;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Vector;

public class CGStatistics {

	final private static String STATSFILENAME = "stats.txt";
	
	final private Vector<Integer> playerScore = new Vector<Integer>();
	final private Vector<Integer> opponentScore = new Vector<Integer>();

	public CGStatistics() {
		;
	}
	
	// save results.
	public boolean saveStats(final int pScore, final int oScore) {
		if (pScore < CGScoreTable.MINIMUM)	return false;
		if (pScore > CGScoreTable.MAXIMUM)	return false;
		if (oScore < CGScoreTable.MINIMUM)	return false;
		if (oScore > CGScoreTable.MAXIMUM)	return false;
		
		loadStats();
		
		playerScore.add(new Integer(pScore));
		opponentScore.add(new Integer(oScore));
		
		if (properSaveStats())	return true;
		else					return false;
	}
	private boolean properSaveStats() {
		if (playerScore.size() != opponentScore.size())	return false;
	    try {
	    	final PrintStream out = new PrintStream(new FileOutputStream(STATSFILENAME));
	    	for(int i=0; i<playerScore.size(); i++) {
	    		out.println(playerScore.elementAt(i) + "\t" + opponentScore.elementAt(i));
	    	}
	    	out.close();
	    	return true;
	    }
	    catch (IOException ex) {
	    	return false;
	    }
	}
	
	// load existing results.
	public boolean loadStats() {
		playerScore.removeAllElements();
		opponentScore.removeAllElements();
		try {
			
			// open the file.
			final BufferedReader in = new BufferedReader(new FileReader(STATSFILENAME));

			class ScoreLine {
				final private static int INVALID = -100;
				final private int player;
				final private int opponent;
				final private boolean isInvalid;
				private ScoreLine(final String line) {
					if (line == null) {
						player = INVALID;
						opponent = INVALID;
						isInvalid = true;
						return;
					}
					if (line.length()<3) {
						player = INVALID;
						opponent = INVALID;
						isInvalid = true;
						return;
					}
					
					// parse the line.
					int tP = INVALID;
					int tO = INVALID;
					String t = "";
					int j;
					for (j=0; j<line.length(); j++) {
						if (line.substring(j, j+1).equals("\t"))	break;
						t = t + line.substring(j, j+1);
					}
					j++;
					if (!(j<line.length())) {
						player = INVALID;
						opponent = INVALID;
						isInvalid = true;
						return;
					}
					
					try {
						tP = Integer.parseInt(t);
					}
					catch (NumberFormatException ex) {
						player = INVALID;
						opponent = INVALID;
						isInvalid = true;
						return;
					}
					t = "";
					for (; j<line.length(); j++) {
						if (line.substring(j, j+1).equals("\t"))	break;
						t = t + line.substring(j, j+1);
					}
					try {
						tO = Integer.parseInt(t);
					}
					catch (NumberFormatException ex) {
						player = INVALID;
						opponent = INVALID;
						isInvalid = true;
						return;
					}
					player = tP;
					opponent = tO;
					isInvalid = false;
				}
			}
			String line;
			while ( (line = in.readLine()) != null) {
				ScoreLine scoreLine = new ScoreLine(line);
				if (scoreLine.isInvalid)	continue;
				playerScore.add(new Integer(scoreLine.player));
				opponentScore.add(new Integer(scoreLine.opponent));
			}

			// close the file.
			in.close();
			
			return true;
		}
		
		// no previous data exist; ignore.
		catch (FileNotFoundException ex) {
			return true;
		}
		
		// fail on read errors.
		catch (IOException ex) {
			return false;
		}
	}
}
