package net.mummila.crosswordgame;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;



// query the player for the score (s)he wishes award to himself.
public class CGSubmitDialog extends Dialog {

	// the dialog title.
	final private static String TITLE_SUBMIT	= "Vahvistus: vuoron siirto";
	final private static String TITLE_SCORE		= "Vahvistus: pisteet ja sanan kiinnitys";
	final private static String TITLE_SCORE_WORD= "Vahvistus: pisteet, sanasto ja sanan kiinnitys";
	final private static String TITLE_WORD		= "Vahvistus: sanasto ja sanan kiinnitys";
	
	// dialog width.
	final private static int WIDTH	= 280;
	
	// the prompt above the score Spinner. will be wrapped.
	final private static String PROMPT_SCORE	=
		"Valitse alla olevalla stimell pistemr jonka " +
		"haluat list pistetilillesi tst siirrosta. " +
		"Stimess valmiiksi oleva lukema on ohjelman tekem arvio, " +
		"eik se vlttmtt ole oikein.\n" +
		"\n" +
		"Monenko pisteen arvoinen tm siirto oli?";

	// the prompt above the word input. will be wrapped.
	final private static String PROMPT_WORD	=
		"Alla on syttmsi sana sellaisena kuin ohjelma sen nkee. " +
		"Sanaa ei lytynyt sanastosta, ja halutessasi voit nyt tallentaa sen sinne. " +
		"Voit mys muokata sanaa ennen sen tallentamista, " +
		"mikli ohjelman tarjoama nkemys siit on virheellinen. " +
		"Sanan on oltava riittvn pitk ennen kuin sen tallentaminen on mahdollista. " +
		"Thn syttmsi sana ei vaikuta pelilaudalla olevaan sanaan.";
	final private static String PROMPT_WORD_INVALID	=
		"Syttmsi sana ei tyttnyt sanoille asetettuja ehtoja. " +
		"Pelin sntjen mukaan yhdell vuorolla sytettyjen kirjainten on oltava " +
		"joko samalla rivill tai samalla sarakkeella, " +
		"eik niiden vliin saa jd tyhji ruutuja. " +
		"Sanojen on lisksi oltava vhintn kaksikirjaimisia. " +
		"Mikli epilet syttmsi sanan rikkoneen sntj, " +
		"voit peruuttaa sen kiinnityksen valitsemalla " +
		"\"Ei\" alla olevista vaihtoehdoista.";
	
	// the word input has four labels:
	// WORD_LABEL_SAVE [_______] WORD_LABEL_INTO
	//   [ ] WORD_LABEL_YES	[x] WORD_LABEL_NO
	//
	// for example:
	// Save [______) into dictionary?
	//       [ ] Yes	[x] No
	final private static String WORD_LABEL_SAVE	= "Tallennetaanko";
	final private static String WORD_LABEL_INTO	= "sanastoon?";
	final private static String WORD_LABEL_YES	= "Kyll";
	final private static String WORD_LABEL_NO	= "Ei";
	
	// reasonable minimum and maximum values for the score Spinner,
	// and two values outside those limits to signal 'Cancel'
	// and 'no points' back to caller.
	// in truth, negative values would not be out of the question for
	// the minimum, but non-negativity is a sad limit imposed by Spinner.
	// MAXIMUM + current score goes beyond CGScoreTable.MAXIMUM when
	// current score > 0, but it's taken care of at the receiving end,
	// so there's no need to worry about setting a proper MAXIMUM here.
	// NONE and CANCEL need to be seen by CGActionButtonSet. 
	final private static int	SCORE_MINIMUM 	= 0;
	final private static int	SCORE_MAXIMUM 	= CGScoreTable.MAXIMUM;
	final public static int		SCORE_NONE		= -1;
	final public static int		SCORE_CANCEL	= -2;

	// an invalid value for an inserted word.
	// this should be crafted so that it's impossible for the
	// player to produce it with the tools that (s)he's given.
	// it mustn't be empty (""), because empty is used to separate
	// cases where no word is needed from cases where a word
	// could not be read from the board.
	final private static String WORD_INVALID	= "_?INVALID?_";
	
	final private static String BUTTONS_PROMPT =
		"Alla olevilla napeilla voit vahvistaa tai peruuttaa sanan kiinnityksen " +
		"sek yll tekemsi valinnat.\n" +
		"\n" +
		"Haluatko kiinnitt syttmsi sanan pelilaudalle " +
		"ja vahvistaa yll tekemsi valinnat?";
	final private static String BUTTONS_PROMPT_EMPTY =
		"Et ole tll kierroksella kyttnyt yhtn kirjainta. " +
		"Haluatko siirt vuoron vastapelaajalle?";
	final private static String BUTTONS_PROMPT_INVALID =
		"Haluatko kiinnitt syttmsi sanan pelilaudalle?";
	
	// the choice Button labels.
	final private static String BUTTON_LABEL_SUBMIT	= "Kyll";
	final private static String BUTTON_LABEL_CANCEL	= "Ei";

	// the score to be returned.
	private int pts;
	
	// the inserted word.
	private String word;
	
	// the dialog window.
	private Shell shell;

	
	
	// the constructor sets the initial score and parent game.
	private final CrosswordGame game;
	public CGSubmitDialog(final int init, final String w, final Shell parent, final CrosswordGame g) {
		super (parent, SWT.NONE);
		if (init == SCORE_NONE)											pts = SCORE_NONE;
		else if ( (SCORE_MINIMUM <= init) && (init <= SCORE_MAXIMUM) )	pts = init;
		else															pts = SCORE_MINIMUM;
		game = g;
		if (w.equals("") || !game.dictionary.isValidWord(w))	word = WORD_INVALID;
		else if (game.dictionary.contains(w))					word = "";
		else													word = w;
	}
	public CGSubmitDialog(final int init, final Shell parent, final CrosswordGame g) {
		super (parent, SWT.NONE);
		if ( (SCORE_MINIMUM <= init) && (init <= SCORE_MAXIMUM) )	pts = init;
		else														pts = SCORE_MINIMUM;
		word = "";
		game = g;
	}
	public CGSubmitDialog(final String w, final Shell parent, final CrosswordGame g) {
		super (parent, SWT.NONE);
		pts = SCORE_NONE;
		game = g;
		if (w.equals("") || !game.dictionary.isValidWord(w))	word = WORD_INVALID;
		else if (game.dictionary.contains(w))					word = "";
		else													word = w;
	}
	public CGSubmitDialog(final Shell parent, final CrosswordGame g) {
		super (parent, SWT.NONE);
		pts = SCORE_NONE;
		game = g;
		word = "";
	}

	
	
	// open the dialog.
	// on 'Submit' returns the score selected by the user.
	// on 'Cancel' returns a non-valid score. 
	public int open() {

		Shell parent = getParent();
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText(getText());
		GridLayout gl = new GridLayout(1, true);
		gl.marginHeight = 20;
		gl.marginWidth = 20;
		shell.setLayout(gl);

		// set driving truth values.
		boolean hasScore = (pts != SCORE_NONE);
		boolean hasWord = !(word.equals(""));
		
		if (hasScore && hasWord)	shell.setText(TITLE_SCORE_WORD);
		else if (hasScore)			shell.setText(TITLE_SCORE);
		else if (hasWord)			shell.setText(TITLE_WORD);
		else						shell.setText(TITLE_SUBMIT);
		
		// score input for modifying the number of points to be awarded for the move.
		final ScoreInput scoreInput;
		if (hasScore) {
			scoreInput = new ScoreInput();
			Label separator1 = new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);
			GridData fill = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
			separator1.setLayoutData(fill);
		}
		else			scoreInput = null;
		
		// text input for modifying the word to be saved in the dictionary.
		final WordInput wordInput;
		if (hasWord) {
			if (word.equals(WORD_INVALID))	wordInput = new WordInput("");
			else							wordInput = new WordInput(word);
			
			Label separator1 = new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);
			GridData fill = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
			separator1.setLayoutData(fill);
		}
		else	wordInput = null;

		
		
		// set the prompt above the choice Buttons.
		// wrapped in a Composite since everything else is too, so that margins are equal.
		class ButtonPrompt extends Composite {
			public ButtonPrompt(final boolean hasScore, final boolean hasWord) {
				super(shell, SWT.NONE);
				GridLayout gl = new GridLayout(1, false);
				this.setLayout(gl);
				final Label prompt = new Label(this, SWT.WRAP);
				if (hasScore)				prompt.setText(BUTTONS_PROMPT);
				else if (hasWord) {
					if (word.equals(""))	prompt.setText(BUTTONS_PROMPT_INVALID);
					else					prompt.setText(BUTTONS_PROMPT);
				}
				else						prompt.setText(BUTTONS_PROMPT_EMPTY);
				GridData gdprompt = new GridData();
				gdprompt.widthHint = WIDTH;
				prompt.setLayoutData(gdprompt);
			}
		}
		new ButtonPrompt(hasScore, hasWord);
		
		final Buttons buttons = new Buttons(scoreInput, wordInput, shell);
		GridData center = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
		buttons.setLayoutData(center);

		// set up an exit listener to make exit act like 'Cancel'.
		// http://www.java2s.com/Code/Java/SWT-JFace-Eclipse/Preventashellfromclosingprompttheuser.htm
	    shell.addListener(SWT.Close, new Listener() {
			public void handleEvent(Event event) {
				pts = SCORE_CANCEL;
			}
	    });

	    shell.pack();
		shell.open();

		Display display = parent.getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) display.sleep();
		}
		
		return pts;		
	}
	
	
	
	// text input for modifying the word to be saved in the dictionary.
	private class WordInput extends Composite implements VerifyListener, ModifyListener {
		final private Text wordText;
		final private SaveSelector saveSelector;
		
		
		
		// the constructor's parameter
		private WordInput(final String word) {
			super(shell, SWT.NONE);
			GridLayout gl = new GridLayout(3, false);
			this.setLayout(gl);
			
			// set the prompt above the score Spinner.
			final Label prompt = new Label(this, SWT.WRAP);
			if (word.equals("")) {
				prompt.setText(PROMPT_WORD_INVALID);
				GridData gdprompt = new GridData();
				gdprompt.horizontalSpan = 3;
				gdprompt.widthHint = WIDTH;
				prompt.setLayoutData(gdprompt);
				saveSelector = null;
				wordText = null;
			}
			else {
				prompt.setText(PROMPT_WORD);
				GridData gdprompt = new GridData();
				gdprompt.horizontalSpan = 3;
				gdprompt.widthHint = WIDTH;
				prompt.setLayoutData(gdprompt);
	
				Label saveLabel = new Label(this, SWT.NONE);
				saveLabel.setText(WORD_LABEL_SAVE);
	
				// a text field for editing the word.
				wordText = new Text(this, SWT.SINGLE | SWT.BORDER);
				wordText.setTextLimit(CGDictionary.DICTIONARY_WORD_MAXSIZE);
				wordText.setText(word);
				GridData gdText = new GridData();
				gdText.widthHint = 120;	// about the width of 15 M's.
				wordText.setLayoutData(gdText);
				
				Label intoLabel = new Label(this, SWT.NONE);
				intoLabel.setText(WORD_LABEL_INTO);
				
				saveSelector = new SaveSelector(this);
				GridData gdSelector = new GridData(GridData.HORIZONTAL_ALIGN_CENTER);
				gdSelector.horizontalSpan = 3;
				saveSelector.setLayoutData(gdSelector);
				if (game.dictionary.isValidWord(word))	enableSave();
				else									disableSave();
				wordText.addVerifyListener(this);
				wordText.addModifyListener(this);
			}
		}
		
		private boolean saveSelected() {
			return saveSelector.saveSelected();
		}
		
		private void disableSave() {
			saveSelector.disable();
			wordText.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_RED));
		}
		private void enableSave() {
			saveSelector.enable();
			wordText.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_GREEN));
		}

		public void verifyText(VerifyEvent event) {
			event.doit = false;
			
			char c = event.character;
			
			// allow backspace and delete. convert to uppercase on-the-fly.
			// http://www.java2s.com/Tutorial/Java/0280__SWT/Allowbackspaceanddelete.htm
			if (c == '\u0008' || c == '\u007F') event.doit = true;
			else if (Character.isLetter(c))	{
				event.text = event.text.toUpperCase();
				event.doit = true;
			}

		}

		public void modifyText(ModifyEvent event) {
			if (game.dictionary.isValidWord(wordText.getText()))	enableSave();
			else													disableSave();
		}
		
		

		// a couple of radio buttons for selecting whether to
		// save the word to the dictionary or not.
		private class SaveSelector extends Composite {
			final private Button yesButton;
			final private Button noButton;
			private SaveSelector(Composite parent) {
				super(parent, SWT.NONE);
				GridLayout glSelector = new GridLayout(2, false);
				this.setLayout(glSelector);
				
				yesButton = new Button(this, SWT.RADIO);
				yesButton.setText(WORD_LABEL_YES);
				noButton = new Button(this, SWT.RADIO);
				noButton.setText(WORD_LABEL_NO);
				noButton.setSelection(true);
			}
			private boolean saveSelected() {
				return yesButton.getSelection();
			}
			private void disable() {
				noButton.setSelection(true);
				yesButton.setSelection(false);
				this.setEnabled(false);
			}
			private void enable() {
				this.setEnabled(true);
			}
		}
	}
	
	
	
	// score input for modifying the number of points to be awarded for the move.
	private class ScoreInput extends Composite {
		final private Spinner score;
		public ScoreInput() {	this(false);	}
		public ScoreInput(boolean invisible) {
			super(shell, SWT.NONE);
			GridLayout gl = new GridLayout(1, true);
			this.setLayout(gl);
			
			// set the prompt above the score Spinner.
			final Label prompt = new Label(this, SWT.WRAP);
			prompt.setText(PROMPT_SCORE);
			GridData gdPrompt = new GridData();
			gdPrompt.widthHint = WIDTH;
			prompt.setLayoutData(gdPrompt);
		
			// the score Spinner.
			// http://www.java2s.com/Code/Java/SWT-JFace-Eclipse/NumberInputDialog.htm
		    score = new Spinner(this, SWT.BORDER);
		    score.setMinimum(SCORE_MINIMUM);
		    score.setMaximum(SCORE_MAXIMUM);
		    score.setSelection(pts);
		    score.setIncrement(1);
		    score.setPageIncrement(10);
			GridData gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.horizontalAlignment = SWT.CENTER;
			score.setLayoutData(gd);
			
			if (invisible)	this.setVisible(false);
		}
		public int getScore() {
			return score.getSelection();
		}
	}

	
	
	// the final choice buttons: 'Submit', 'Cancel'.
	private class Buttons extends Composite {
		private Buttons(final ScoreInput score, final WordInput wordInput, final Shell shell) {
			super(shell, SWT.NONE);
			GridLayout gl = new GridLayout(2, true);
			this.setLayout(gl);
			
			GridData gdFill = new GridData(GridData.FILL_BOTH);
			
		    final Button submit = new Button(this, SWT.PUSH);
			submit.setText(BUTTON_LABEL_SUBMIT);
			submit.setLayoutData(gdFill);
			
			gdFill = new GridData(GridData.FILL_BOTH);
			
			final Button cancel = new Button(this, SWT.PUSH);
			cancel.setText(BUTTON_LABEL_CANCEL);
			cancel.setLayoutData(gdFill);

			// react to buttons being clicked.
			submit.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					pts = (score == null) ? SCORE_NONE : score.getScore();
					if (!word.equals("")) {
						if (wordInput != null) {
							if (wordInput.saveSelected()) {
								game.dictionary.add(word);
								game.dictionary.save();
							}
						}
					}
					shell.dispose();
				}
			});
			cancel.addListener(SWT.Selection, new Listener() {
				public void handleEvent(Event event) {
					pts = SCORE_CANCEL;
					shell.dispose();
				}
			});
		}
	}	
}

